//#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <stdint.h>
#include <vector>
#include <pcl/console/parse.h>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <pcl/io/ply_io.h>

#include <pcl/io/obj_io.h>
#include <pcl/io/vtk_lib_io.h>
#include <pcl/PolygonMesh.h>

using namespace std;

int
main(int argc, char** argv)
{

	pcl::PointCloud<pcl::PointXYZ> cloud;
	pcl::PointXYZ p;
	ifstream ins;
	// offset posotion of laser range finder
	double offset = 230.0;
	
	int interval = 1;
	pcl::console::parse_argument(argc, argv, "-i", interval);

	double maxdepth = 0;
	pcl::console::parse_argument(argc, argv, "-d", maxdepth);

	bool ply = false;
	ply = pcl::console::find_switch(argc, argv, "--ply");

	bool display_help = false;
	display_help = pcl::console::find_switch(argc, argv, "--help");

	if (display_help || argc <= 1)
	{
		std::cout << "Usage:depth2pcd.exe [OPTION]... [ptx]" << std::endl;
		std::cout << "  -i          interval" << std::endl;
		std::cout << "  -d          max depth" << std::endl;
		std::cout << "     --ply    save as ply file" << std::endl;
		std::cout << "     --help   display this help and exit" << std::endl;

		return (0);
	}

	std::cout << "Interval  : " << interval << std::endl;
	std::cout << "Maximum depth : " << maxdepth << std::endl;
	std::cout << "Save as ply file : " << ply << std::endl;

	std::vector<int> pcd_indices;
	pcd_indices = pcl::console::parse_file_extension_argument(argc, argv, ".txt");

	if (pcd_indices.size() < 1) {
		std::cout << "ptx file must be specified" << std::endl;
		return (0);
	}

	// ********************* t@C̓ǂݍ *********************
	std::string infile = argv[pcd_indices[0]];
	std::string outfile = argv[pcd_indices[0]];

	if (ply)
		outfile.replace(outfile.length() - 3, 3, "ply");
	else
		outfile.replace(outfile.length() - 3, 3, "pcd");

	ins.open(infile.c_str());
	
	std::vector<int> depth;

	if (!ins)
	{
		// Opening a file encounters a problem
		cerr << "Error: file could not be opened!" << endl;
		exit(1);
	}
	else
	{
		while (!ins.eof())
		{
			int d;
			ins >> d;
			depth.push_back(d);
		}
	}
	ins.close();

	int height = 760;
	int width = depth.size() / height;
	double  mm2m = 0.001;
	
	double endAngle = -85.0 * M_PI / 180.0;
	double startAngle = 105.0 * M_PI / 180.0;

	int n = 0;
	double hdiv = -2.0 * M_PI / width;
	double vdiv = (endAngle - startAngle) / height;

	// for partial images
	if (width < 3750) hdiv = -2.0 * M_PI / 3750.0;
	
	for (int h = 0; h < height; h++){
		for (int w = 0; w < width; w++){
			p.x = (double)(depth[n] * cos(vdiv*h + startAngle) + offset) * cos(hdiv*w) * mm2m;
			p.y = (double)(depth[n] * cos(vdiv*h + startAngle) + offset) * sin(hdiv*w) * mm2m;
			p.z = (double)depth[n] * sin(vdiv*h + startAngle) * mm2m;

			if (n++%interval) continue;
			if (maxdepth > 0.0 && sqrt(p.x*p.x + p.y*p.y + p.z*p.z) > maxdepth) continue;
			if (p.x != 0.0 || p.y != 0.0 || p.z != 0.0) cloud.points.push_back(p);

		}
	}

	cloud.width = cloud.points.size();
	cloud.height = 1;
	cloud.points.resize(cloud.width*cloud.height);

	if (ply){
		bool binary = true;
		bool use_camera = false;
		pcl::PLYWriter w;
		w.write(outfile, cloud, binary, use_camera);
	}
	else {
		pcl::io::savePCDFileASCII(outfile, cloud);
	}

	return 0;
}
